﻿using Infragistics.Controls.Grids.Primitives;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace RefreshGroupSummaries
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        SampleData data = new SampleData();

        public MainWindow()
        {
            InitializeComponent();

            xamGrid1.ItemsSource = data;

            // update data values from background thread.
            BackgroundWorker bw = new BackgroundWorker();
            bw.DoWork += (s, e) =>
            {
                Random random = new Random();

                while (true)
                {
                    List<int> updatedIndexes = new List<int>();

                    for (int i = 0; i < 50; i++)
                    {
                        int index = random.Next(0, data.Count);
                        updatedIndexes.Add(index);

                        data[index].salary = random.NextDouble() * 10000;
                    }

                    // updating the UI needs to happen on the main thread.
                    Dispatcher.BeginInvoke(new Action(() =>
                    {
                        foreach (int index in updatedIndexes)
                            data[index].OnPropertyChanged("salary");
                    }));
                    
                    Thread.Sleep(100);
                }
            };
            bw.RunWorkerAsync();

            DispatcherTimer timer = new DispatcherTimer();
            timer.Interval = TimeSpan.FromMilliseconds(50);
            timer.Tick += timer_Tick;
            timer.Start();
        }

        int groupToUpdate = 0;
        void timer_Tick(object sender, EventArgs e)
        {
            if (xamGrid1.GroupBySettings.GroupByColumns.Count > 0)
            {
                // refresh the current group
                (xamGrid1.Rows[groupToUpdate] as GroupByRow).Rows[0].Manager.RefreshSummaries();

                // move on to the next group
                groupToUpdate++;

                // make sure it doesn't go past the last group
                if (groupToUpdate >= xamGrid1.Rows.Count)
                    groupToUpdate = 0;
            }
        }
    }

    public class SampleDataItem : INotifyPropertyChanged
    {
        #region INotifyPropertyChanged Implementation
        public event PropertyChangedEventHandler PropertyChanged;
        public void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        public string name { get; set; }
        public string department { get; set; }
        public double salary { get; set; }
        public string email { get; set; }
    }

    public class SampleData : ObservableCollection<SampleDataItem>
    {
        public SampleData()
        {
            for (int i = 0; i < 1000; i++)
            {
                if (i <= 333)
                    Add(new SampleDataItem { name = "group 1", department = "Sales", salary = 70000.0, email = "joe@email.com" });
                else if (i > 333 && i <= 666)
                    Add(new SampleDataItem { name = "group 2", department = "Sales", salary = 70000.0, email = "joe@email.com" });
                else
                    Add(new SampleDataItem { name = "group 3", department = "Sales", salary = 70000.0, email = "joe@email.com" });
            }
        }
    }
}
